# -*- coding: iso-8859-15 -*-
###
# Copyright (c) 2009-2012, Markus Rennings
# All rights reserved.
#
# Fr Folkland Radio, Radios, die daraus entstanden sind und Personen, die
# daran in 2008/2009 beteiligt waren, ist dieses Plugin nicht lizensiert. Somit
# besteht fr diese Personengruppe und Radios keine Erlaubnis, dieses Plugin zu
# benutzen.
#
# If you can't read/understand the German text above, don't worry, you're not
# included in that exception.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#   * Redistributions of source code must retain the above copyright notice,
#     this list of conditions, and the following disclaimer.
#   * Redistributions in binary form must reproduce the above copyright notice,
#     this list of conditions, and the following disclaimer in the
#     documentation and/or other materials provided with the distribution.
#   * Neither the name of the author of this software nor the name of
#     contributors to this software may be used to endorse or promote products
#     derived from this software without specific prior written consent.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
###

import re
import time

import supybot.utils as utils
from supybot.commands import *
import supybot.plugins as plugins    # __IGNORE_WARNING__ (eric5)
import supybot.ircutils as ircutils    # __IGNORE_WARNING__ (eric5)
import supybot.callbacks as callbacks
import supybot.ircmsgs as ircmsgs
import supybot.registry as registry    # __IGNORE_WARNING__ (eric5)
import supybot.conf as conf    # __IGNORE_WARNING__ (eric5)
import supybot.schedule as schedule
import supybot.world as world


def replace_entities(text):
    """
    Replaces HTML "Entities" with readable characters

    Input:  String
    Return: String
    """
    entities = {"&#x26;": "&",
                "&#x27;": "'",
                "&#x92;": "'",
                "&#xB4;": "'",
                "&#xC1;": "",
                "&#xE1;": "",
                "&#xE2;": "",
                "&#xE4;": "",
                "&#xE9;": "",
                "&#xEB;": "",
                "&#xED;": "",
                "&#xEE;": "",
                "&#xF1;": "",
                "&#xF3;": "",
                "&#xF6;": "",
                "&#xFA;": "",
                "&#xFC;": "",
                "&#xFF;": ""
                }

    text = text.replace(
        '&#Xbe;&#Xed;&#X05;&#X08;P&#Xe2;&#Xb6;&#Xb6;&#Xc4;&#Xed;&#X05;&#X08;P&#X',
        " ...")
    for k in entities:
        text = text.replace(k, entities[k])
    return text


class Shoutcast(callbacks.Plugin):
    """This plugin contains some commands to fetch informations from a running
       shoutcast server
    """
    threaded = True

    ShoutcastRE = re.compile("""^.*? (?P<KBPS>\d+)\skbps\ with\ (?P<HOERER>\d+)
                              \ of\ (?P<MAXHOERER>\d+)
                              .*?\((?P<UNIQUE>\d+)
                              .*?Peak:\ (?P<PEAK>\d+)
                              .*?Stream\ Title:\ (?P<SENDUNG>.*?)
                              \ Content.*?Current\ Song:\ (?P<TITEL>.*?)
                              \ Written\ by""", re.S|re.M|re.X)


    def __init__(self, irc):
        self.__parent = super(Shoutcast, self)
        self.__parent.__init__(irc)
        self._current_song = ""
        self._data = {}
        self._periodic_name = "ShoutcastUpdater"
        self._periodic_time = 0
        self._event_created = False
        self.updatePeriodicAnnouncer(irc)
        
    def die(self):
        """
        Make sure, the periodicEvent is removed
        """
        self.__parent.die()
        try:
            schedule.removePeriodicEvent(self._periodic_name)
        except KeyError:
            self.log.info(utils.str.normalizeWhitespace("""Error removing 
                    periodic event: Existed one actually? 
                    UpdateIntervall==0?"""))

    ###########################################################################
    ### helper functions
    ###########################################################################
    # pylint: disable-msg=C0103
    # In Supybot methods with uppercase Letters in its name are not visible in
    # the channel, therefore using camelCase here and disabling the linter's
    # info

    def getShoutcast(self):
        """
        Gets the informations from the Shoutcast server

        Return: on success: { title (str), listener (str), maxlistener (str),
                              peak {str}, program (str), Error (bool) }
                on error:   { Error (bool) }
        """

        try:
            text = utils.web.htmlToText(utils.web.getUrl(self.registryValue(
                                                                "StreamURL")))
        except:
            self.log.debug(utils.str.normalizeWhitespace("""Error retrieving 
                    the page from the shoutcast server"""))
            return {"Error": True}

        result = self.ShoutcastRE.match(text)
        daten = {}
        if result:
            daten = {"listener": int(result.group('HOERER')),
                      "maxlistener": int(result.group('MAXHOERER')),
                      "peak": int(result.group('PEAK')),
                      "unique": int(result.group('UNIQUE')),
                      "kbps": int(result.group('KBPS')),
                      "program": replace_entities(result.group('SENDUNG')),
                      "title": replace_entities(result.group('TITEL')),
                      "Error": False
            }
        else:
            daten = {"Error": True}

        return daten



    def getLastShoutcast(self):
        """
        Gets the informations about the last played songs.

        Return: [ "current song", "current-1 Song", "current-2 Song", ... ]
        """

        raw = ""
        try:
            raw = utils.web.getUrl(self.registryValue("StreamURL") +
                                   "played.html")
            raw = re.sub("^.*Song Title.*?<td>", '', raw)
            raw = re.sub("</table></br>.*$", '', raw)
            songlist = raw.split("</tr><tr>")
            songlist.pop()   # Das letzte Element entfernen, da es nur das
                             # Shoutcast-Copyright enthlt
            for i in range(len(songlist)):
                songlist[i] = utils.web.htmlToText(songlist[i])
                # Don't remove the timestamps as shoutcast doesn't reset
                # that list if the stream goes offline.
                #songlist[i] = sub("\d+:\d+:\d+\s", '', songlist[i])
                # The above line removes the timestamp (if wanted)
                songlist[i] = re.sub("\sWritten by Stephen.*", '', songlist[i])
            songlist[0] = re.sub("Current Song", "(Aktuelles Lied)",
                                 songlist[0])
        except:
            songlist = ""
        return songlist



    def getDataRetrErrorMsg(self,  msg):
        """
        Gets the error message configured in supybot.plugins.Shoutcast.dataRetrError,
        replaces @@station if needed and returns the complete error msg.
        """
        antwort = self.registryValue("dataRetrError",  msg.args[0])
        if antwort.count("@@station@@"):
            antwort = antwort.replace("@@station@@", self.registryValue("Name"))
        return antwort

    ###########################################################################
    ### Announcer
    ###########################################################################

    def updatePeriodicAnnouncer(self, irc):
        """
        Sets up/updates/removes the periodic event for the announcer and the
        updater
        """
        if self.registryValue("UpdateIntervall") == self._periodic_time:
            # The interval time in config didn't change, so no need to update
            # the periodic Event
            return

        if self._event_created:
            # PeriodicEvent already created, so delete it before creating it new
            schedule.removePeriodicEvent(self._periodic_name)
            self._event_created = False
            self.log.info("Event %s deleted" % self._periodic_name)

        if self.registryValue("UpdateIntervall") > 0:
            self._periodic_time = self.registryValue("UpdateIntervall")
            # a time is set in the config so create the periodic event
            schedule.addPeriodicEvent(lambda: self.Announcer(irc),
                                      self._periodic_time, self._periodic_name)
            self.log.info("PeriodicAnnouncer created, update intervall is %i" %
                          self._periodic_time)
            self._event_created = True



    def Announcer(self, irc):
        """
        The announcer; Updates the data and if the song changed announces it to
        every channel which is set to True in config. The text to announce is
        configurable through config channel plugins.Shoutcast.AnnounceText.
        """
        if self._periodic_time != self.registryValue("UpdateIntervall"):
            self.updatePeriodicAnnouncer(irc)
        self._data = self.getShoutcast()
        current_song = self._data["title"]
        if current_song != self._current_song:
            self._current_song = current_song
            timestamp = time.strftime(self.registryValue("timestampFormat"))
            for Irc in world.ircs:
                for channel in Irc.state.channels:
                    if self.registryValue("Announce", channel):
                        to_ann = self.registryValue("AnnounceText", channel)
                        to_ann = to_ann.replace("@@SONG@@", current_song)
                        to_ann = to_ann.replace("@@STREAM@@",
                                                self.registryValue("Name"))
                        to_ann = to_ann.replace("@@TIME@@", timestamp)
                        Irc.queueMsg(ircmsgs.privmsg(channel, to_ann))



    def doPrivmsg(self, irc, msg):    # IGNORE:W0613
        """
        Catches every privmsg and checks if the UpdateIntervall has changed by
        calling UpdatePeriodicAnnouncer.
        This is to be sure we don't miss a change in the interval time because
        it was set too long earlier
        """
        self.updatePeriodicAnnouncer(irc)




    # pylint: enable-msg=C0103
    # enabling linter info about uppercase letters in method names as below
    # should only be user commands (callable from irc) which MUST be in all
    # lower case

    # pylint: disable-msg=W0613
    # disable warnings about unused arbuments, as in the most methods below here
    # msg and args are not used but applied when calling one of the methods
    ###########################################################################
    ### commands
    ###########################################################################

    def stream(self, irc, msg, args):
        """takes no arguments

        Send the MRL to the channel with which you can listen to the stream
        """

        antwort = self.registryValue("streamText",  msg.args[0])
        if antwort.count("@@station@@"):
            antwort = antwort.replace("@@station@@",  self.registryValue("Name"))
        if antwort.count("@@mrl@@"):
            antwort = antwort.replace("@@mrl@@",  self.registryValue("StreamURL"))
        irc.reply(utils.str.normalizeWhitespace(antwort))
    stream = wrap(stream)



    def all(self, irc, msg, args):
        """takes no arguments

        Displays all informations about the stream
        """

        if not self.registryValue("ShowAll", msg.args[0]):
            antwort = self.registryValue("notActivated")
        else:
            if (self._periodic_time == 0) or (self._periodic_time > 5):
                self._data = self.getShoutcast()
            if self._data["Error"]:
                antwort = self.getDataRetrErrorMsg(msg)
            else:
                antwort = self.registryValue("allText",  msg.args[0])
                if antwort.count("@@station@@"):
                    antwort = antwort.replace("@@station@@", self.registryValue("Name"))
                if (self.registryValue("ShowListeners", msg.args[0]) and antwort.count("@@listeners@@")):
                    antwort = antwort.replace("@@listeners@@", str(self._data["listener"]))
                if (self.registryValue("ShowMaxListeners", msg.args[0]) and antwort.count("@@maxlisteners")):
                    antwort = antwort.replace("@@maxlisteners@@", str(self._data["maxlistener"]))
                if (self.registryValue("ShowUnique", msg.args[0]) and antwort.count("@@unique@@")):
                    antwort = antwort.replace("@@unique@@", str(self._data["unique"]))
                if (self.registryValue("ShowTitle", msg.args[0]) and antwort.count("@@title@@")):
                    antwort = antwort.replace("@@title@@",  self._data["title"])
                if (self.registryValue("ShowKbps", msg.args[0]) and antwort.count("@@kbps@@")):
                    antwort = antwort.replace("@@kbps@@", str(self._data["kbps"]))
                if (self.registryValue("ShowPeak", msg.args[0]) and antwort.count("@@peak@@")):
                    antwort = antwort.replace("@@peak@@", str(self._data["peak"]))
                if (self.registryValue("ShowProgram", msg.args[0]) and antwort.count("@@program@@")):
                    antwort = antwort.replace("@@program@@", self._data["program"])
        irc.reply(utils.str.normalizeWhitespace(antwort))
    all = wrap(all)



    def title(self, irc, msg, args):
        """takes no arguments

        Displays the current song.
        """

        if (self._periodic_time == 0) or (self._periodic_time > 5):
            self._data = self.getShoutcast()
        if not self._data["Error"]:
            antwort = self.registryValue("titleText", msg.args[0])
            if antwort.count("@@station@@"):
                antwort = antwort.replace("@@station@@",  self.registryValue("Name"))
            if antwort.count("@@title@@"):
                antwort = antwort.replace("@@title@@",  self._data["title"])
        else:
            antwort = self.getDataRetrErrorMsg(msg)
        irc.reply(utils.str.normalizeWhitespace(antwort))
    title = wrap(title)



    def listeners(self, irc, msg, args):
        """takes no arguments

        Displays the current listeners count.
        """

        if not self.registryValue("ShowListeners", msg.args[0]):
            antwort = self.registryValue("notActivated")
        else:
            if (self._periodic_time == 0) or (self._periodic_time > 5):
                self._data = self.getShoutcast()
            if not self._data["Error"]:
                antwort = self.registryValue("listenersText",  msg.args[0])
                if antwort.count("@@listeners@@"):
                    antwort = antwort.replace("@@listeners@@", str(self._data["listener"]))
                if (antwort.count("@@maxlisteners@@") and self.registryValue("ShowMaxListeners",  msg.args[0])):
                    antwort = antwort.replace("@@maxlisteners@@", str(self._data["maxlistener"]))
                if (antwort.count("@@unique@@") and self.registryValue("ShowUnique",  msg.args[0])):
                    antwort = antwort.replace("@@unique@@", str(self._data["unique"]))
                if antwort.count("@@station@@"):
                    antwort = antwort.replace("@@station@@", self.registryValue("Name"))
            else:
                antwort = self.getDataRetrErrorMsg(msg)
        irc.reply(utils.str.normalizeWhitespace(antwort))
    listeners = wrap(listeners)



    def program(self, irc, msg, args):
        """takes no arguments

        Displays the name of the current program.
        """

        if not self.registryValue("ShowProgram", msg.args[0]):
            antwort = self.registryValue("notActivated")
        else:
            if (self._periodic_time == 0) or (self._periodic_time > 5):
                self._data = self.getShoutcast()
            if not self._data["Error"]:
                antwort = self.registryValue("programText", msg.args[0])
                if antwort.count("@@station@@"):
                    antwort = antwort.replace("@@station@@",  self.registryValue("Name"))
                if antwort.count("@@program@@"):
                    antwort = antwort.replace("@@program@@",  self._data["program"])
            else:
                antwort = self.getDataRetrErrorMsg(msg)
        irc.reply(utils.str.normalizeWhitespace(antwort))
    program = wrap(program)



    def peak(self, irc, msg, args):
        """takes no arguments

        Displays the listener's peak.
        """

        if not self.registryValue("ShowPeak", msg.args[0]):
            antwort = self.registryValue("notActivated")
        else:
            if (self._periodic_time == 0) or (self._periodic_time > 5):
                self._data = self.getShoutcast()
            if not self._data["Error"]:
                antwort = self.registryValue("peakText",  msg.args[0])
                if antwort.count("@@station@@"):
                    antwort = antwort.replace("@@station@@",  self.registryValue("Name"))
                if antwort.count("@@peak@@"):
                    antwort = antwort.replace("@@peak@@",  self._data["peak"])
            else:
                antwort = self.getDataRetrErrorMsg(msg)
        irc.reply(utils.str.normalizeWhitespace(antwort))
    peak = wrap(peak)



    def maxlisteners(self, irc, msg, args):
        """takes no arguments

        Displays the available listener slots.
        """

        if not self.registryValue("ShowMaxListeners", msg.args[0]):
            antwort = self.registryValue("notActivated")
        else:
            if (self._periodic_time == 0) or (self._periodic_time > 5):
                self._data = self.getShoutcast()
            if not self._data["Error"]:
                antwort = self.registryValue("maxListenersText",  msg.args[0])
                if antwort.count("@@maxlisteners@@"):
                    antwort = antwort.replace("@@maxlisteners@@",  str(self._data["maxlistener"]))
                if antwort.count("@@station@@"):
                    antwort = antwort.replace("@@station@@",  self.registryValue("Name"))
            else:
                antwort = self.getDataRetrErrorMsg(msg)
        irc.reply(utils.str.normalizeWhitespace(antwort))
    maxlisteners = wrap(maxlisteners)



    def last(self, irc, msg, args):
        """takes no arguments

        Displays the last played songs.
        """

        if not self.registryValue("ShowLast", msg.args[0]):
            antwort = self.registryValue("notActivated")
        else:
            songlist = self.getLastShoutcast()

            if songlist:
                antwort = self.registryValue("lastText", msg.args[0])
                if antwort.count("@@station@@"):
                    antwort = antwort.replace("@@station@@", self.registryValue("Name"))
                if antwort.count("@@last@@"):
                    songs = ""
                    for i in range(len(songlist)):
                        songs = "%s %s || " % (songs, songlist[i])
                    songs = utils.str.normalizeWhitespace(songs.rstrip(" || "))
                    antwort = antwort.replace("@@last@@", songs)
            else:
                antwort = self.getDataRetrErrorMsg(msg)
        irc.reply(utils.str.normalizeWhitespace(antwort))
    last = wrap(last)



    def homepage(self, irc, msg, args):
        """takes no arguments

        Displays the URL of the homepage of the station.
        """

        antwort = self.registryValue("homepageText", msg.args[0])
        if antwort.count("@@station@@"):
            antwort = antwort.replace("@@station@@", self.registryValue("Name"))
        if antwort.count("@@url@@"):
            antwort = antwort.replace("@@url@@", self.registryValue("Homepage"))
        irc.reply(utils.str.normalizeWhitespace(antwort))
    homepage = wrap(homepage)


Class = Shoutcast    # IGNORE:C0103

# vim:set shiftwidth=4 tabstop=4 expandtab textwidth=79 foldmethod=indent:
